# Datadog Extension
<!-- status autogenerated section -->
| Status        |           |
| ------------- |-----------|
| Stability     | [alpha]  |
| Distributions | [contrib] |
| Issues        | [![Open issues](https://img.shields.io/github/issues-search/open-telemetry/opentelemetry-collector-contrib?query=is%3Aissue%20is%3Aopen%20label%3Aextension%2Fdatadog%20&label=open&color=orange&logo=opentelemetry)](https://github.com/open-telemetry/opentelemetry-collector-contrib/issues?q=is%3Aopen+is%3Aissue+label%3Aextension%2Fdatadog) [![Closed issues](https://img.shields.io/github/issues-search/open-telemetry/opentelemetry-collector-contrib?query=is%3Aissue%20is%3Aclosed%20label%3Aextension%2Fdatadog%20&label=closed&color=blue&logo=opentelemetry)](https://github.com/open-telemetry/opentelemetry-collector-contrib/issues?q=is%3Aclosed+is%3Aissue+label%3Aextension%2Fdatadog) |
| Code coverage | [![codecov](https://codecov.io/github/open-telemetry/opentelemetry-collector-contrib/graph/main/badge.svg?component=extension_datadog)](https://app.codecov.io/gh/open-telemetry/opentelemetry-collector-contrib/tree/main/?components%5B0%5D=extension_datadog&displayType=list) |
| [Code Owners](https://github.com/open-telemetry/opentelemetry-collector-contrib/blob/main/CONTRIBUTING.md#becoming-a-code-owner)    | [@jackgopack4](https://www.github.com/jackgopack4), [@dineshg13](https://www.github.com/dineshg13), [@mx-psi](https://www.github.com/mx-psi), [@songy23](https://www.github.com/songy23) |

[alpha]: https://github.com/open-telemetry/opentelemetry-collector/blob/main/docs/component-stability.md#alpha
[contrib]: https://github.com/open-telemetry/opentelemetry-collector-releases/tree/main/distributions/otelcol-contrib
<!-- end autogenerated section -->

The Datadog Extension enables collector configuration and build info to be viewed in the Datadog Infrastructure Monitoring application. Specifically, if host info matches between Datadog Exporter and Datadog Extension, collector configuration may be viewed in Host List or in Resource Catalog on any host.

You may configure this extension in service, using the following configuration values:
`api::key`: a Datadog API Key, required
`api::site`: your Datadog site value (e.g. us5.datadoghq.com), defaults to "datadoghq.com"
`hostname`: custom hostname; if you do not specify one, the extension will try to infer one. Note: If set, this must match the `hostname` value set in the Datadog Exporter, if also enabled in your collector. If `hostname` is left unset in both Datadog Exporter and Datadog Extension, the inferred hostname will match correctly.

## Local HTTP Server Functionality

The Datadog Extension includes a local HTTP server that exposes the OpenTelemetry Collector's configuration and metadata. This server is intended to make it easier to inspect and debug the collector's state locally, and to view the collector configuration and metadata that is intended to be displayed in the Datadog app.

**Purpose:**
- Display collector configuration and metadata in the Datadog Infrastructure Monitoring app. More announcements and UI changes will be made soon regarding this matter.

**How it works:**
- The extension starts a server (by default on `localhost:9875` at the `/metadata` path) that provides information regarding the collector's configuration, active components, build info, and other metadata.
- This information is also sent to the Datadog backend for infrastructure monitoring and fleet automation features.

**Example configuration:**
```yaml
extensions:
  datadog:
    api:
      key: <YOUR_DATADOG_API_KEY>
      site: "datadoghq.com"
    http:
      endpoint: "localhost:9875"
      path: "/metadata"
```

## Enabling Proxy Support

You can set up proxy configuration for outbound requests to Datadog by following the example below:

**Example configuration with proxy:**
```yaml
extensions:
  datadog:
    api:
      key: <YOUR_DATADOG_API_KEY>
      site: "datadoghq.com"
    proxy_url: "http://proxy.example.com:8080"
    timeout: 30s
    tls:
      insecure_skip_verify: true
```
- `proxy_url`: URL of the HTTP proxy to use for outbound requests.
- `timeout`: Timeout for HTTP requests to Datadog.
- `tls`: TLS settings for outbound connections.

## Notes
- The extension is in active development. Functionality and configuration options may change as Datadog infrastructure monitoring features evolve.
- For the latest updates and UI changes, watch the [Datadog Monitor blog](https://www.datadoghq.com/blog/) and the [OpenTelemetry Collector Contrib repository](https://github.com/open-telemetry/opentelemetry-collector-contrib).
